## CS&SS 569 - Visualizing Data and Models
## TA: Ramses Llobet - UW, Department of Political Science
## Lab Week 1: Intermediate R (data management, and analysis)

rm(list = ls()) # clean memory


## All the shortcuts

# Press Option + SHIFT + K (Mac) 
# or, ALT + SHIFT + K (Windows)




# Some Useful RStudio Shortcuts (for Windows)

# Press "CTRL + ENTER" to run code (either current line or selection)
# Press "CTRL + SHIFT + C" to comment/ uncomment code
# Press "CTRL + L" to clear the console
# Press "ALT + -" to insert assignment operator ( <- )
# Press "CTRL + SHIFT + M" to insert pipe operator ( |>  )






#### data wrangling:-------------------------------------------------

rm(list = ls()) # clean memory


# This is how generally set up the first lines (preamble) of all my script files:

setwd(dirname(rstudioapi::getActiveDocumentContext()$path)) # To set current folder as your wd.
print(getwd()) # Check that your wd is correct
dir() # Check the file on your directory

# Followed by the list of packages that I will use.

library(stargazer)
library(tidyverse)



# load data
df <- read_csv(file = "data/gapminder.csv")


# tibble (tbl) is a special class of data frame
class(df)

print(df)




# Chaining code with pipes using dplyr is a way to create a more readable 
# and efficient code when working with data frames. The basic idea is to use
# the |>  operator to pass the output of one function as the input to the 
# next function, without having to save the intermediate steps as separate 
# objects.


# Basic data wrangling

df |> 
  count(country)


count(df, country) # Equivalent to df |>  count(country)

# with base R:
arrange(count(df, country), n)

# with dplyr:
df |> 
  count(country) |> 
  arrange(n) # Rather than: arrange(count(df, country), n)



# Basic data wrangling: `filter()`

# Extract rows that meet logical criteria

df |> 
  filter(country == "Brazil")


df |> 
  filter(
    country == "Brazil" | country == "Russia (Soviet Union)" | 
      country == "India"  | country == "China"
  ) |>  print(n=50)


df |> 
  select(country, year, gdpPercap)


USAdata <- 
  df |> 
  filter(country == "United States",
         year %in% 1980:2010) |> 
  select(year, gdpPercap)

USAdata # check data


# Basic data wrangling: `summarize()`

USAdata |> 
  summarize(avg_gdpPercap = mean(gdpPercap))


# Basic data wrangling: `group_by()` & `summarize()`

# Create a grouped version of the table with `group_by()`

df |> 
  group_by(country) |> 
  summarize(avg_gdpPercap = mean(gdpPercap)) |> 
  arrange(desc(avg_gdpPercap)) |>  
  print(n=50)


# Basic data wrangling: `mutate()`

df |>  
  mutate(
    row = row_number(),
    #  %/% operator is an integer division.
    decade = year %/% 10 * 10
    # It calculates the quotient of a division operation and 
    # discards the decimal part
  ) |> 
  select(row, country, year, decade, gdpPercap)


# What if we want to know countries' average GDP per capita over decades? 

df |> 
  mutate(decade = year %/% 10 * 10) |> 
  group_by(country, decade) |> 
  summarize(decAvg_gdp = mean(gdpPercap)) |>  
  print(n=50)


# Basic data wrangling: long and wide format

df_long <-
  df |> 
  mutate(decade = year %/% 10 * 10) |> 
  group_by(country, decade) |> 
  summarize(decAvg_gdp = mean(gdpPercap))

# In "long" format:
# - each column is a variable
# - each row is an observation

df_long

# From long to wide:

df_long |>  
  pivot_wider(id_cols = "country", names_from = "decade", values_from = "decAvg_gdp")


# From wide to long

df_wide <-
  df_long |>  
  pivot_wider(id_cols = "country", names_from = "decade", values_from = "decAvg_gdp")



df_wide |>  
  pivot_longer(cols = `1950`:`2000`, names_to = "deacde", values_to = "gpd")


## load new data


pop <- read_csv("data/pop.csv")

names(pop)

# Compare with econ
names(df)


# merge data using "joint"

df <-
  df |>  
  left_join(pop, by=c("country","year")) |>  
  na.omit()


## Let's create a new categorical variable based on population

# cutpoints by the quartiles of the population

Qts <- quantile(df$pop, prob = c(0.25, 0.75), na.rm = TRUE)
print(Qts)


Q1 <- Qts[1]
Q3 <- Qts[2]

df <- 
  df |> 
  mutate(popCat = case_when(pop < Q1 ~ "low",
                            pop >= Q1 & pop < Q3 ~ "middle",
                            pop > Q3 ~ "high")) 

## create a dichotomous variable of country income level

df <-
  df |>  
  mutate(wealth = ifelse(gdpPercap > mean(gdpPercap),1,0),
         wealth = factor(wealth,
                         levels = c(0,1),
                         labels = c("low_income","high_income")))



#### Analysis:------------------------------------------------------


rm(list = ls()) # clean memory

library(tidyverse)

# Let's use iris data

head(iris)

tail(iris)


df <- as_tibble(iris)

summary(df)

names(df)

str(df)


# Ways to select variables of interest:

df[1,]  # First row

df[,1]  # First column




# You can also select columns by their names:

names(df)

cor(df[,c("Sepal.Length","Sepal.Width","Petal.Length","Petal.Width")])

correlations <- cor(df[,c("Sepal.Length","Sepal.Width","Petal.Length","Petal.Width")])

round(correlations, digits = 2)


# too many decimals!

# select() and filter() from dyplr:

select(df,Sepal.Length,Sepal.Width,Petal.Length)

df3 <- filter(df, Species == "setosa")


# Too much code and replicated objects? Better call pipe (operator)!

df |>  
  select(Sepal.Length,Sepal.Width) |>  
  cor() |>  
  round(digits = 2)


# Interested in the within-group correlation? Check it out

df |>  
  group_by(Species) |>  
  summarize(Length_Width = cor(Sepal.Length,Sepal.Width,use="complete.obs"))


# Let's visualize those correlations, time to ggplot!

?ggplot

df |>  
  ggplot(mapping = aes(x = Petal.Length,
                       y = Sepal.Length)) +
  geom_point() +
  geom_smooth(method = "lm", formula = "y~ x + I(x^2)")

# In ggplot, use the function "geom_smooth" to fit linear relationships


# Run a naive model 

lm1 <- lm(Sepal.Length ~ Species + Sepal.Width + I(Sepal.Width^2) + log(Petal.Length), data = iris)

# Summary

summary(lm1)


# Suppose our variable of interest is Sepal.Width. To simulate its effect:
## 1. we manipulate Sepal.Width to be from 1 to 10
## 2. we hold other continuous variables at their means
## 3. we choose a particular level for factor/categorical variable (can't average a factor)







scenario <- 
  df |>                     # from original data
  summarize_all(mean) |>    # summarize the means for most variables (can't average Species)
  uncount(10) |>            # repeat the row 10 times
  mutate( 
    Sepal.Width = 1:10,     # create hypothetical values for Sepal.Width
    Species = "setosa"      # set Species to be "setosa"
  ) 






## Check if all hypothetical scenarios make sense
scenario

## Predict average outcomes
lm1_pred <- predict(lm1, 
                    newdata = scenario, 
                    interval = "confidence")

## A bit of wrangling before visualization
lm1_pred <- 
  lm1_pred |> 
  as_tibble() |> 
  mutate(Sepal.Width = 1:10)

## Visualize 
ggplot(lm1_pred, 
       mapping = aes(x = Sepal.Width, 
                     y = fit, 
                     ymin = lwr, 
                     ymax = upr)) +
  geom_ribbon(alpha = 0.2) +
  geom_line() 


# Let's save the visual!

?ggsave

ggsave("output/plots/ris_quadratic.pdf",
       width = 6,
       height = 3)


# Save the df of interest:

write.csv(scenario, file = "output/data/Iris_Pred.csv", row.names = F)

save(scenario, file = "output/data/Iris_Pred.Rdata")



##### tinytex for pdf files:-----------------------------------------------------------

# To work with R Markdown, you must install tinytex to render pdf documents.

install.packages("tinytex")

tinytex::install_tinytex() # install tiny text only once! comment the code afterwards



########################################################
